#!/bin/sh
# Copyright (c) 2012 William Hubbs <w.d.hubbs@gmail.com>
# Released under the 2-clause BSD license.

. "/lib/rc/sh/functions.sh"

if ! mountinfo -q -f tmpfs "/lib/rc/init.d"; then
	einfo "The OpenRC dependency data has already been migrated."
	exit 0
fi

if [ ! -d "/run" ]; then
	eerror "'/run' is not a directory."
	eerror "This means the OpenRC dependency data cannot be migrated."
	eerror "Please create the '/run' directory and reboot the system."
	exit 1
fi

if ! mountinfo -q -f tmpfs "/run"; then
	for x in "/run/."* "/run/"*; do
		case "$x" in
			"/run/."|"/run/..")
				continue
				;;
		esac
		if [ -e "$x" ]; then
				eerror "Your '/run' directory contains files."
				eerror "Please reboot the system."
				exit 1
		fi
	done

	mount -t tmpfs -o mode=0755,nosuid,nodev \
		tmpfs "/run" 2> /dev/null
	if [ $? != 0 ]; then
		eerror "Unable to mount a tmpfs on '/run'."
		eerror "This means the OpenRC dependency data cannot be migrated."
		eerror "Please create the '/run' directory and reboot the system."
		exit 1
	fi
fi

rm -rf "/run/openrc"
cp -a "/lib/rc/init.d" "/run/openrc"
rc-update -u
rm -rf "/lib/rc/init.d"
umount "/lib/rc/init.d"
einfo "The OpenRC dependency data was migrated successfully."
exit 0
